@@ -21,7 +21,6 @@
 (require 'helm)
 (require 'helm-types)
 (require 'helm-utils)
-(require 'helm-elscreen)
 (require 'helm-grep)
 (require 'helm-regexp)
 (require 'helm-help)
@@ -142,6 +141,11 @@ Only buffer names are fuzzy matched when this is enabled,
   "Face for buffer file names in `helm-buffers-list'."
   :group 'helm-buffers-faces)
 
+(defface helm-non-file-buffer
+    '((t (:inherit italic)))
+  "Face used for non-file buffers in `helm-buffers-list'."
+  :group 'helm-buffers-faces)
+
 
 ;;; Buffers keymap
 ;;
@@ -159,7 +163,7 @@ Only buffer names are fuzzy matched when this is enabled,
     (define-key map (kbd "C-c =")     'helm-buffer-run-ediff)
     (define-key map (kbd "M-=")       'helm-buffer-run-ediff-merge)
     (define-key map (kbd "C-=")       'helm-buffer-diff-persistent)
-    (define-key map (kbd "M-U")       'helm-buffer-revert-persistent)
+    (define-key map (kbd "M-G")       'helm-buffer-revert-persistent)
     (define-key map (kbd "C-c d")     'helm-buffer-run-kill-persistent)
     (define-key map (kbd "M-D")       'helm-buffer-run-kill-buffers)
     (define-key map (kbd "C-x C-s")   'helm-buffer-save-persistent)
@@ -223,6 +227,7 @@ Only buffer names are fuzzy matched when this is enabled,
    (keymap :initform helm-buffer-map)
    (migemo :initform 'nomultimatch)
    (volatile :initform t)
+   (nohighlight :initform t)
    (resume :initform (lambda () (setq helm-buffers-in-project-p nil)))
    (help-message :initform 'helm-buffer-help-message)))
 
@@ -362,11 +367,11 @@ See `ido-make-buffer-list' for more infos."
          (helm-buffer--show-details
           name name-prefix file-name size mode dir
           'helm-buffer-file 'helm-buffer-process nil details 'filebuf))
-        ;; Any non--file buffer.=>grey italic
+        ;; Any non--file buffer.=>italic
         (t
          (helm-buffer--show-details
           name (and proc name-prefix) dir size mode dir
-          'italic 'helm-buffer-process proc details 'nofile))))))
+          'helm-non-file-buffer 'helm-buffer-process proc details 'nofile))))))
 
 (defun helm-highlight-buffers (buffers _source)
   "Transformer function to highlight BUFFERS list.
@@ -393,11 +398,15 @@ Should be called after others transformers i.e (boring buffers)."
         ;; The max length of a number should be 1023.9X where X is the
         ;; units, this is 7 characters.
         for formatted-size = (and size (format "%7s" size))
-        collect (cons (if helm-buffer-details-flag
-                          (concat truncbuf "\t" formatted-size
-                                  "  " fmode "  " meta)
-                        name)
-                      (get-buffer i))))
+        collect           (let ((helm-pattern (helm-buffers--pattern-sans-filters
+                                               (and helm-buffers-fuzzy-matching ""))))
+                            (cons (if helm-buffer-details-flag
+                                      (concat
+                                       (funcall helm-fuzzy-matching-highlight-fn truncbuf)
+                                       "\t" formatted-size
+                                       "  " fmode "  " meta)
+                                      (funcall helm-fuzzy-matching-highlight-fn name))
+                                  (get-buffer i)))))
 
 (defun helm-buffer--get-preselection (buffer)
   (let ((bufname (buffer-name buffer)))
@@ -423,12 +432,21 @@ Should be called after others transformers i.e (boring buffers)."
       (helm-update preselect))))
 (put 'helm-toggle-buffers-details 'helm-only t)
 
-(defun helm-buffers-sort-transformer (candidates _source)
+(defun helm-buffers--pattern-sans-filters (&optional separator)
+  (cl-loop for p in (helm-mm-split-pattern helm-pattern)
+           unless (member (substring p 0 1) '("*" "/" "@" "!"))
+           collect p into lst
+           finally return (mapconcat 'identity lst (or separator " "))))
+
+(defun helm-buffers-sort-transformer (candidates source)
   (if (string= helm-pattern "")
       candidates
-    (sort candidates
-          (lambda (s1 s2)
-              (< (string-width s1) (string-width s2))))))
+      (if helm-buffers-fuzzy-matching
+          (let ((helm-pattern (helm-buffers--pattern-sans-filters)))
+            (funcall helm-fuzzy-sort-fn candidates source))
+          (sort candidates
+                (lambda (s1 s2)
+                  (< (string-width s1) (string-width s2)))))))
 
 (defun helm-buffers-mark-similar-buffers-1 ()
   (with-helm-window
@@ -486,11 +504,18 @@ i.e same color."
                 (and pos pos-test)
                 (and neg neg-test (not neg-test)))))))
 
-(defun helm-buffer--match-pattern (pattern candidate)
+(defvar helm-buffer--memo-hash (make-hash-table :test 'equal))
+(defun helm-buffer--memo-pattern (pattern)
+  (or (gethash pattern helm-buffer--memo-hash)
+      (puthash pattern (helm--mapconcat-pattern pattern)
+               helm-buffer--memo-hash)))
+
+(defun helm-buffer--match-pattern (pattern candidate &optional nofuzzy)
   (let ((bfn (if (and helm-buffers-fuzzy-matching
+                      (not nofuzzy)
                       (not helm-migemo-mode)
                       (not (string-match "\\`\\^" pattern)))
-                 #'helm--mapconcat-pattern
+                 #'helm-buffer--memo-pattern
                  #'identity))
         (mfn (if helm-migemo-mode
                  #'helm-mm-migemo-string-match #'string-match)))
@@ -503,32 +528,33 @@ i.e same color."
   (let* ((cand (replace-regexp-in-string "^\\s-\\{1\\}" "" candidate))
          (buf  (get-buffer cand))
          (regexp (cl-loop with pattern = helm-pattern
-                          for p in (split-string pattern)
+                          for p in (helm-mm-split-pattern pattern)
                           when (string-match "\\`\\*" p)
                           return p)))
     (if regexp
         (when buf
           (with-current-buffer buf
-            (let ((mjm (format-mode-line mode-name)))
+            (let ((mjm (symbol-name major-mode)))
               (helm-buffer--match-mjm regexp mjm))))
         t)))
 
 (defun helm-buffers--match-from-pat (candidate)
-  (let ((regexp-list (cl-loop with pattern = helm-pattern
-                              for p in (split-string pattern)
-                              unless (string-match
-                                      "\\`\\(\\*\\|/\\|@\\)" p)
-                              collect p)))
+  (let* ((regexp-list (cl-loop with pattern = helm-pattern
+                               for p in (helm-mm-split-pattern pattern)
+                               unless (string-match
+                                       "\\`\\(\\*\\|/\\|@\\)" p)
+                               collect p))
+         (nofuzzy (cdr regexp-list)))
     (if regexp-list
         (cl-loop for re in regexp-list
-                 always (helm-buffer--match-pattern re candidate))
+                 always (helm-buffer--match-pattern re candidate nofuzzy))
         t)))
 
 (defun helm-buffers--match-from-inside (candidate)
   (let* ((cand (replace-regexp-in-string "^\\s-\\{1\\}" "" candidate))
          (buf  (get-buffer cand))
          (regexp (cl-loop with pattern = helm-pattern
-                          for p in (split-string pattern)
+                          for p in (helm-mm-split-pattern pattern)
                           when (string-match "\\`@\\(.*\\)" p)
                           return (match-string 1 p))))
     (if (and buf regexp)
@@ -545,7 +571,7 @@ i.e same color."
          (buf  (get-buffer cand))
          (buf-fname (buffer-file-name buf))
          (regexps (cl-loop with pattern = helm-pattern
-                          for p in (split-string pattern)
+                          for p in (helm-mm-split-pattern pattern)
                           when (string-match "\\`/" p)
                           collect p)))
     (if regexps
@@ -714,13 +740,6 @@ If REGEXP-FLAG is given use `query-replace-regexp'."
     (helm-exit-and-execute-action 'switch-to-buffer-other-frame)))
 (put 'helm-buffer-switch-other-frame 'helm-only t)
 
-(defun helm-buffer-switch-to-elscreen ()
-  "Run switch to elscreen  action from `helm-source-buffers-list'."
-  (interactive)
-  (with-helm-alive-p
-    (helm-exit-and-execute-action 'helm-find-buffer-on-elscreen)))
-(put 'helm-buffer-switch-to-elscreen 'helm-only t)
-
 (defun helm-buffer-run-ediff ()
   "Run ediff action from `helm-source-buffers-list'."
   (interactive)
