@@ -235,6 +235,15 @@ Behave differently depending of `helm-selection' (current candidate in helm-buff
 - candidate is a file         => open it.
 - marked candidates (1+)      => open them with default action.
 
+Note that when copying, renaming etc... from `helm-find-files' you
+will have a file completion with `helm-read-file-name' to select the
+destination file; To not confuse users of `read-file-name' or
+`read-directory-name' RET behave normally, it exit the minibuffer as
+soon as you press RET, if you want the same behavior as in
+`helm-find-files', bind `helm-ff-RET' to the `helm-read-file-map':
+
+    (define-key helm-read-file-map (kbd \"RET\") 'helm-ff-RET)
+
 *** Find file at point
 
 Helm is using `ffap' partially or completely to find file at point
@@ -455,6 +464,11 @@ and are not renamed to current directory, IOW use this (\\#) to rename files ins
 In the second prompt (replace regexp with) shortcut for `upcase', `downcase' and `capitalize'
 are available, respectively `%u', `%d' and `%c'.
 
+*** Edit marked files in a dired buffer
+
+You can open a dired buffer with only marked files with `\\<helm-find-files-map>\\[helm-ff-run-marked-files-in-dired]'
+With a prefix arg you can open this same dired buffer in wdired mode for editing files.
+
 *** Copying renaming asynchronously
 
 If you use async library (if you have installed helm from MELPA you do) you can enable
